'==================================================================================================
' Language: VBScript
' Title: ZonePlayer UniqueDeviceName Finder
' Author: Osler
' Description: This script will search for UPnP devices matching the ZonePlayer type and populate a
' HouseBot Device (Name: "ZonePlayer_<ZoneName>"; Property: "UDN") with the UniqueDeviceName of the
' ZonePlayer corresponding to the ZoneName. This script should be used for intial device discovery
' (i.e., when initially setting up the Sonos system with HouseBot or when adding an additional
' ZonePlayer to an existing set-up.
'==================================================================================================

Option Explicit

Dim oZonePlayerFinder
Dim cZonePlayers
Dim oDeviceObject
Dim sUDN()
Dim oZonePlayer()
Dim DeviceCounter
Dim oZonePlayer_DeviceProperties()
Dim sZoneName()
Dim DeviceCheck
DeviceCheck = MsgBox("Have you configured the ZonePlayer_<ZoneName> Execute Program Device(s) as required?", vbYesNo, "ZonePlayer_Discovery")

CheckIt()
Set oZonePlayerFinder = CreateObject("UPnP.UPnPDeviceFinder")
Obtain_UDN()
Set oZonePlayerFinder = Nothing
Call MsgBox("ZonePlayer_Discovery completed. Use this device again when adding or deleting ZonePlayers to your system.", vbOKOnly, "ZonePlayer_Discovery")

Private Sub Obtain_UDN()
	Set cZonePlayers = oZonePlayerFinder.FindByType( "urn:schemas-upnp-org:device:ZonePlayer:1", 0 )
	If cZonePlayers.Count = 0 Then
		Exit Sub
	End If
	Dim ReturnValue
	Dim EmptyArg(0)
	Dim sZoneAttributes(1)
	ReDim oZonePlayer(cZonePlayers.Count)
	ReDim sUDN(cZonePlayers.Count)
	ReDim oZonePlayer_DeviceProperties(cZonePlayers.Count)
	ReDim sZoneName(cZonePlayers.Count)
	DeviceCounter = 1

	For Each oDeviceObject In cZonePlayers
		sUDN(DeviceCounter) = oDeviceObject.UniqueDeviceName
		DeviceCounter = DeviceCounter + 1
	Next

	Dim I
	For I = 1 To cZonePlayers.Count
		Set oZonePlayer(I) = cZonePlayers.Item(sUDN(I))
		Set oZonePlayer_DeviceProperties(I) = oZonePlayer(I).Services("urn:upnp-org:serviceId:DeviceProperties")
		ReturnValue = oZonePlayer_DeviceProperties(I).InvokeAction("GetZoneAttributes", EmptyArg, sZoneAttributes)
		sZoneName(I) = sZoneAttributes(0)
		SetDeviceUDN(I)
	Next
End Sub

Private Sub SetDeviceUDN(ZonePlayerNumber)
Dim HBServer
Dim szRSP
Dim Device
Dim UDN

Set HBServer = CreateObject("HBControlMod.HBControl")

szRSP = HBServer.Connect(2525, "127.0.0.1", "")

Device = "ZonePlayer_" & sZoneName(ZonePlayerNumber)
UDN = sUDN(ZonePlayerNumber)

Call HBServer.SetPropertyValue(Device, "ZoneName", sZoneName(ZonePlayerNumber))
Call HBServer.SetPropertyValue(Device, "UDN", UDN)
Call HBServer.SetPropertyValue("ZonePlayer_Discovery", "Execute Program", "No")
Set HBServer = Nothing
End Sub

Private Sub CheckIt()
	If DeviceCheck = 6 Then
		Call MsgBox("Device discovery may take up to 9 seconds, depending on the number of UPnP compliant devices attached.", vbOKOnly, "ZonePlayer_Discovery")
		Exit Sub
	ElseIf DeviceCheck = 7 Then
		Call MsgBox("Please create the Execute Program Device(s) for each zone of your Sonos system before continuing.", vbOKOnly, "ZonePlayer_Discovery")
		WScript.Quit
	End If
End Sub
