Option Explicit

'==========================================================================================================
' UPnP Variables
'==========================================================================================================
Dim oZonePlayerFinder
Dim sUDN
Dim oZonePlayer
Dim oZonePlayer_MediaRenderer
Dim oZonePlayer_AVTransport
Dim oZonePlayer_RenderingControl
'==========================================================================================================
' Current ZonePlayer Status Variables
'==========================================================================================================
Dim CurrentTransportState
Dim CurrentVolumeLevel
Dim CurrentTrebleLevel
Dim CurrentBassLevel
Dim CurrentMuteStatus
Dim CurrentLoudnessStatus
Dim CurrentTrack
Dim CurrentTrackLength
Dim CurrentMetaData
Dim CurrentTrackPosition
'==========================================================================================================
' Input ZonePlayer Command Variables Obtained from the HouseBot Device "ZonePlayer_<ZoneName>"
'==========================================================================================================
Dim NewTransportState
Dim NewVolumeLevel
Dim NewTrebleLevel
Dim NewBassLevel
Dim NewMuteStatus
Dim NewLoudnessStatus

Dim ErrorCount
'=========================================================================================================
' Create the UPnPDeviceFinder and then call the need subroutines.
'=========================================================================================================
Set oZonePlayerFinder = CreateObject("UPnP.UPnPDeviceFinder")
GetDeviceValues() 'Get the property values from HouseBot
Zone_GetStatus(sUDN) 'Get the current settings of the ZonePlayer
Zone_ExecuteCommands(sUDN) 'Compare the property values from HouseBot to the ZonePlayer settings - change if different
Zone_GetTrackInfo(sUDN) 'Get the current track information
Zone_GetStatus(sUDN) 'Get the update settings of the ZonePlayer
SetDeviceValues() 'Set the property values in HouseBot to reflect the settings of the ZonePlayer
Set oZonePlayerFinder = Nothing

'==========================================================================================================
' Status Checking Subroutine: Queueries ZonePlayer for current settings.
'==========================================================================================================
Private Sub Zone_GetStatus(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	Set oZonePlayer_RenderingControl = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:RenderingControl")
	
	Dim ReturnValue
	Dim TransportInfo(2)
	Dim InstanceID(0)
	Dim VolumeInput(1)
	Dim LoudnessInput(1)
	Dim MuteInput(1)
	Dim CurrentBass(0)
	Dim CurrentTreble(0)
	Dim CurrentVolume(0)
	Dim CurrentLoudness(0)
	Dim CurrentMute(0)
	Dim Channel
	InstanceID(0) = 0
	Channel = "Master"
	
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("GetTransportInfo", InstanceID, TransportInfo)
	
	If TransportInfo(0) = "PLAYING" Then
		CurrentTransportState = "Play"
	ElseIf TransportInfo(0) = "STOPPED" Then
		CurrentTransportState = "Stop"
	ElseIf TransportInfo(0) = "PAUSED_PLAYBACK" Then
		CurrentTransportState = "Pause"
	End If
	
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("GetBass", InstanceID, CurrentBass)
	CurrentBassLevel = CStr(CurrentBass(0))
	
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("GetTreble", InstanceID, CurrentTreble)
	CurrentTrebleLevel = CStr(CurrentTreble(0))
	
	VolumeInput(0) = InstanceID(0)
	VolumeInput(1) = Channel
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("GetVolume", VolumeInput, CurrentVolume)
	CurrentVolumeLevel = CStr(CurrentVolume(0))
	
	LoudnessInput(0) = InstanceID(0)
	LoudnessInput(1) = Channel
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("GetLoudness", LoudnessInput, CurrentLoudness)
	CurrentLoudnessStatus = CStr(CurrentLoudness(0))
	
	MuteInput(0) = InstanceID(0)
	MuteInput(1) = Channel
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("GetMute", MuteInput, CurrentMute)
	CurrentMuteStatus = CStr(CurrentMute(0))
End Sub

'========================================================================================================
' Command Execution Subroutine: Compares property values from HouseBot to the current values of the
' ZonePlayer and changes the ZonePlayer value to match the HouseBot value if they differ.
'========================================================================================================
Private Sub Zone_ExecuteCommands(UniqueDeviceName)
	If NewTransportState = "" Then Exit Sub
	If NewVolumeLevel = "" Then Exit Sub
	If NewTrebleLevel = "" Then Exit Sub
	If NewBassLevel = "" Then Exit Sub
	If NewMuteStatus = "" Then Exit Sub
	If NewLoudnessStatus = "" Then Exit Sub
	
	If NewTransportState <> CurrentTransportState Then
		If NewTransportState = "Play" Then
			Call Zone_Play(UniqueDeviceName)
		ElseIf NewTransportState = "Stop" Then
			Call Zone_Stop(UniqueDeviceName)
		ElseIf NewTransportState = "Pause" Then
			Call Zone_Pause(UniqueDeviceName)
		ElseIf NewTransportState = "Next" Then
			Call Zone_Next(UniqueDeviceName)
		ElseIf NewTransportState = "Previous" Then
			Call Zone_Previous(UniqueDeviceName)
		ElseIf NewTransportState = "Next Section" Then
			Call Zone_NextSection(UniqueDeviceName)
		ElseIf NewTransportState = "Previous Section" Then
			Call Zone_PreviousSection(UniqueDeviceName)
		Else
			Exit Sub
		End If
	End If
	
	If NewVolumeLevel <> CurrentVolumeLevel Then
		If NewVolumeLevel > 100 Then
			NewVolumeLevel = 100
		End If
		If NewVolumeLevel < 0 Then
			NewVolumeLevel = 0
		End If
		Call Zone_SetVolume(UniqueDeviceName)
	End If
	
	If NewTrebleLevel <> CurrentTrebleLevel Then
		If NewTrebleLevel > 10 Then
			NewTrebleLevel = 10
		End If
		If NewTrebleLevel > 0 Then
			NewTrebleLevel = 0
		End If
		Call Zone_SetTreble(UniqueDeviceName)
	End If
	
	If NewBassLevel <> CurrentBassLevel Then
		If NewBassLevel > 10 Then
			NewBassLevel = 10
		End If
		If NewBassLevel > 0 Then
			NewBassLevel = 0
		End If
		Call Zone_SetBass(UniqueDeviceName)
	End If
	
	If NewMuteStatus <> CurrentMuteStatus Then
		If NewMuteStatus <> "True" And NewMuteStatus <> "False" Then
			Exit Sub
		End If
		Call Zone_SetMute(UniqueDeviceName)
	End If
	
	If NewLoudnessStatus <> CurrentLoudnessStatus Then
		If LoudnessStatus <> "True" And NewLoudnessStatus <> "False" Then
			Exit Sub
		End If
		Call Zone_SetLoudness(UniqueDeviceName)
	End If
End Sub
		
'==========================================================================================================
' Transport Control Subroutines
'==========================================================================================================
Private Sub Zone_Play(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(1) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	Dim Speed 
	InstanceID = 0 
	Speed = 1 
	PlaySettings(0) = InstanceID 
	PlaySettings(1) = Speed
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("Play", PlaySettings, EmptyArg)
	
	Dim Info(2)
	Dim Instance(0)
	Instance(0) = 0
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("GetTransportInfo", Instance, Info)
	If Info(0) <> "PLAYING" Then
		Zone_Play(sUDN)
		Call ErrorCounter()
		If ErrorCount = 2 Then
			ErrorCount = 0
			Exit Sub
		End If
	End If
End Sub

Private Sub Zone_Stop(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("Stop", PlaySettings, EmptyArg)
	
	Dim Info(2)
	Dim Instance(0)
	Instance(0) = 0
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("GetTransportInfo", Instance, Info)
	If Info(0) <> "STOPPED" Then
		Zone_Stop(sUDN)
		Call ErrorCounter()
		If ErrorCount = 2 Then
			ErrorCount = 0
			Exit Sub
		End If
	End If
End Sub

Private Sub Zone_Pause(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("Pause", PlaySettings, EmptyArg)
	
	Dim Info(2)
	Dim Instance(0)
	Instance(0) = 0
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("GetTransportInfo", Instance, Info)
	If Info(0) <> "PAUSED_PLAYBACK" Then
		Zone_Pause(sUDN)
		Call ErrorCounter()
		If ErrorCount = 2 Then
			ErrorCount = 0
			Exit Sub
		End If
	End If
End Sub

Private Sub Zone_Next(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("Next", PlaySettings, EmptyArg)
End Sub

Private Sub Zone_NextSection(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("NextSection", PlaySettings, EmptyArg)
End Sub

Private Sub Zone_Previous(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("Previous", PlaySettings, EmptyArg)
End Sub

Private Sub Zone_PreviousSection(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim EmptyArg(0)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("PreviousSection", PlaySettings, EmptyArg)
End Sub

'===========================================================================================================
' Rendering Control Subroutines
'===========================================================================================================
Private Sub Zone_SetVolume(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_RenderingControl = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:RenderingControl")
	
	Dim ReturnValue
	Dim InstanceID
	Dim Channel
	Dim VolumeInput(2)
	Dim EmptyArg(0)
	InstanceID = 0
	Channel = "Master"
	VolumeInput(0) = InstanceID
	VolumeInput(1) = Channel
	VolumeInput(2) = NewVolumeLevel
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("SetVolume", VolumeInput, EmptyArg)
End Sub

Private Sub Zone_SetTreble(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_RenderingControl = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:RenderingControl")
	
	Dim ReturnValue
	Dim InstanceID
	Dim TrebleInput(1)
	Dim EmptyArg(0)
	InstanceID = 0
	TrebleInput(0) = InstanceID
	TrebleInput(1) = NewTrebleLevel
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("SetTreble", TrebleInput, EmptyArg)
End Sub

Private Sub Zone_SetBass(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_RenderingControl = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:RenderingControl")
	
	Dim ReturnValue
	Dim InstanceID
	Dim BassInput(1)
	Dim EmptyArg(0)
	InstanceID = 0
	BassInput(0) = InstanceID
	BaseInput(1) = NewBassLevel
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("SetBass", BassInput, EmptyArg)
End Sub

Private Sub Zone_SetMute(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_RenderingControl = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:RenderingControl")
	
	Dim ReturnValue
	Dim InstanceID
	Dim Channel
	Dim MuteInput(2)
	Dim EmptyArg(0)
	InstanceID = 0
	Channel = "Master"
	MuteInput(0) = InstanceID
	MuteInput(1) = Channel
	MuteInput(2) = NewMuteStatus
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("SetMute", MuteInput, EmptyArg)
End Sub

Private Sub Zone_SetLoudness(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_RenderingControl = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:RenderingControl")
	
	Dim ReturnValue
	Dim InstanceID
	Dim Channel
	Dim LoudnessInput(2)
	Dim EmptyArg(0)
	InstanceID = 0
	Channel = "Master"
	LoudnessInput(0) = InstanceID
	LoudnessInput(1) = Channel
	LoudnessInput(2) = NewLoudnessStatus
	ReturnValue = oZonePlayer_RenderingControl.InvokeAction("SetLoudness", MuteInput, EmptyArg)
End Sub

'========================================================================================================
' Currently Play Track Information Subroutine
'========================================================================================================
Private Sub Zone_GetTrackInfo(UniqueDeviceName)
	Set oZonePlayer = oZonePlayerFinder.FindByUDN(UniqueDeviceName)
	Set oZonePlayer_MediaRenderer = oZonePlayer.Children.Item(UniqueDeviceName & "_MR")
	Set oZonePlayer_AVTransport = oZonePlayer_MediaRenderer.Services("urn:upnp-org:serviceId:AVTransport")
	
	Dim PlaySettings(0) 
	Dim TrackInfo(7)
	Dim ReturnValue
	Dim InstanceID 
	InstanceID = 0 
	PlaySettings(0) = InstanceID 
	ReturnValue = oZonePlayer_AVTransport.InvokeAction("GetPositionInfo", PlaySettings, TrackInfo)
	CurrentTrack = TrackInfo(0)
	CurrentTrackLength = TrackInfo(1)
	CurrentMetaData = TrackInfo(2)
	CurrentTrackPosition = TrackInfo(4)
End Sub

Private Sub ErrorCounter()
	ErrorCount = ErrorCount + 1
End Sub

'======================================================================================================
' HouseBot Subroutines
'======================================================================================================

Private Sub GetDeviceValues()
Dim HBServer
Dim szRSP

Set HBServer = CreateObject("HBControlMod.HBControl")

szRSP = HBServer.Connect(2525, "127.0.0.1", "")
sUDN = HBServer.GetPropertyValue("ZonePlayer_Living Room", "UDN")
NewTransportState = HBServer.GetPropertyValue("ZonePlayer_Living Room", "ZPTransportState")
NewVolumeLevel = HBServer.GetPropertyValue("ZonePlayer_Living Room", "ZPVolume")
NewTrebleLevel = HBServer.GetPropertyValue("ZonePlayer_Living Room", "ZPTreble")
NewBassLevel = HBServer.GetPropertyValue("ZonePlayer_Living Room", "ZPBass")
NewMuteStatus = HBServer.GetPropertyValue("ZonePlayer_Living Room", "ZPMuteStatus")
NewLoudnessStatus = HBServer.GetPropertyValue("ZonePlayer_Living Room", "ZPLoudnessStatus")

Set HBServer = Nothing
End Sub

Private Sub SetDeviceValues()
Dim HBServer
Dim szRSP

Set HBServer = CreateObject("HBControlMod.HBControl")

szRSP = HBServer.Connect(2525, "127.0.0.1", "")

Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPTransportState", CurrentTransportState)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPVolume", CurrentVolumeLevel)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPBass", CurrentBassLevel)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPTreble", CurrentTrebleLevel)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPMuteStatus", CurrentMuteStatus)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPLoudnessStatus", CurrentLoudnessStatus)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPCurrentTrack", CurrentTrack)
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPTrackLength", CStr(CurrentTrackLength))
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "ZPTrackPosition", CStr(CurrentTrackPosition))
Call HBServer.SetPropertyValue("ZonePlayer_Living Room", "Execute Program", "No")
Set HBServer = Nothing
End Sub


